; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register										
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0										
; WDP3 WDP2 WDP1 WDP0 Time														
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms									
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms									
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms									
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s									
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s									
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s									
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s									
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s									
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s									
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s									
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 							
; und dann das System neu gestartet												
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MCUSR  MCU Status Register													
; 7,6,5,JTRF,WDRF,BORF,EXTRF,PORF												
; NEUSTART ber WDR, wenn BORF,EXTRF,PORF gestzt ist							
NEUSTART_MIT_WDR:
	in temp, MCUSR
	andi temp,0b00000111
	tst temp
	brne NEUSTART_MIT_WDR_FEHLER
	ret
NEUSTART_MIT_WDR_FEHLER:
	out MCUSR,NULL
	rjmp NEUSTART_MIT_WDR_FEHLER
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Diese Routine wird bei WDR-Zeitberschreitung aufgerufen, hier knnte man		
; sich z.B. den Fehler anzeigen lasen, oder DATEN in den EEPROM schreiben		
WDT:
	WDR
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB an	
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	call wait_100ms
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	call wait_100ms
	rjmp WDT
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	cbi (TESTA_PORT),(TESTA_PIN_NR)					; TEST-PIN-LOW		
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_FEHLER_SCHWER
	sbi (TESTA_PORT),(TESTA_PIN_NR)					; TEST-PIN-HIGH		
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_FEHLER_SCHWER:
	CLI											; INTERRUPTS sperren	
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)		; GELB aus  
	call wait_100ms
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	rjmp PROGRAMM_CHECK_FEHLER_SCHWER
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	; auf AUSGANG								
	sbi (LED_GRUEN_DDR),(LED_GRUEN_PIN_NR)			; GRN		
	sbi (LED_GELB_DDR),(LED_GELB_PIN_NR)			; GELB		
	sbi (LED_ROT_DDR),(LED_ROT_PIN_NR)				; ROT		
	sbi (TESTA_DDR),(TESTA_PIN_NR)					; TEST-PIN	
	sbi (TESTB_DDR),(TESTB_PIN_NR)					; TEST-PIN	
	; LED an									
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN an   
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an	
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  an	

	cbi (TESTA_PORT),(TESTA_PIN_NR)					; TEST-A-PIN-LOW		
	sbi (TESTA_PORT),(TESTA_PIN_NR)					; TEST-A-PIN-HIGH		

	cbi (TESTB_PORT),(TESTB_PIN_NR)					; TEST-B-PIN-LOW		
	sbi (TESTB_PORT),(TESTB_PIN_NR)					; TEST-B-PIN-HIGH		


ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_TEST:
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB aus	
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  aus	

	call wait_100ms
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN an   
	call wait_100ms
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an	
	call wait_100ms
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  an	
	call wait_100ms

	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)			; GRN aus	
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB aus	
	sbi (LED_ROT_PORT),(LED_ROT_PIN_NR)				; ROT  aus	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_LEBENSLICHT:
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB an	
	call wait_500ms
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR)			; GELB aus	
	call wait_500ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STOP_PROGRAMM:
	WDR
rjmp STOP_PROGRAMM
; ##############################################################################
; ##############################################################################
; ##############################################################################
; FEHLER_CODE=temp1																
; FEHLER_WERT=temp2																
; AUFBAU:																		
; 8 BYTE pro DATENSATZ															
; <CODE><WERT><TAG><MONAT><JAHR><hh><mm><ss>									
FEHLERSPEICHER_WRITE_TEMP1_TEMP2:
	push YL
	push YH
	push temp9
	; SCHLEIFE		
	ldi ZL, LOW(adr_FEHLER_SPEICHER80+8*8)
	ldi ZH,HIGH(adr_FEHLER_SPEICHER80+8*8)
	mov YL,ZL
	mov YH,ZH
	adiw YL,8
	ldi temp9,9
FEHLERSPEICHER_WRITE_TEMP1_TEMP2_s:
	rcall FEHLERSPEICHER_WRITE_TEMP1_TEMP2_SCHIEBEN
	subi ZL,16
	subi YL,16
	dec temp9
	brne FEHLERSPEICHER_WRITE_TEMP1_TEMP2_s

	pop temp9
	pop YH
	pop YL
;-------------------------------------------------------------------------------
FEHLERSPEICHER_WRITE_TEMP1_TEMP2_ACTION:
	ldi ZL, LOW(adr_FEHLER_SPEICHER80)
	ldi ZH,HIGH(adr_FEHLER_SPEICHER80)

	ST Z+,temp1
	ST Z+,temp2
	LDS temp,(adr_TAG)
	ST Z+,temp
	LDS temp,(adr_MONAT)
	ST Z+,temp
	LDS temp,(adr_JAHR)
	ST Z+,temp
	LDS temp,(adr_hh)
	ST Z+,temp
	LDS temp,(adr_mm)
	ST Z+,temp
	LDS temp,(adr_ss)
	ST Z+,temp
ret
;-------------------------------------------------------------------------------
FEHLERSPEICHER_WRITE_TEMP1_TEMP2_SCHIEBEN:
	LD temp,Z+	; 1
	ST Y+,temp
	LD temp,Z+	; 2
	ST Y+,temp
	LD temp,Z+	; 3
	ST Y+,temp
	LD temp,Z+	; 4
	ST Y+,temp
	LD temp,Z+	; 5
	ST Y+,temp
	LD temp,Z+	; 6
	ST Y+,temp
	LD temp,Z+	; 7
	ST Y+,temp
	LD temp,Z+	; 8
	ST Y+,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
